"""
default exception
"""
je_editor_error: str = "JEditor error"
"""
init exception
"""
je_editor_init_error: str = "JEditor failed to initialize"
je_editor_init_exec_manager_exception: str = "JEditor failed to initialize the program manager"
"""
exec exception
"""
je_editor_exec_error: str = "JEditor execution error"
file_not_fond_error: str = "File not found"
compiler_not_found_error: str = "Compiler not found"
"""
shell exception
"""
je_editor_shell_error: str = "JEditor shell execution error"
"""
file exception
"""
je_editor_save_file_error: str = "JEditor failed to save file"
je_editor_open_file_error: str = "JEditor failed to open file"
"""
json exception
"""
cant_reformat_json_error: str = "Cannot reformat JSON: is the type correct?"
wrong_json_data_error: str = "Failed to parse JSON"
cant_find_json_error: str = "Can't find JSON file"
cant_save_json_error: str = "Can't save JSON file"
"""
content data error
"""
content_set_compiler_error: str = "Error setting compiler using content"
je_editor_content_file_error: str = "JEditor content file error"
